@@ -1,6 +1,7 @@
 """Plot topologies using networkx and matplotlib."""
 
 import os
+import argparse
 from typing import Any, cast, Dict, Optional
 
 import numpy as np
@@ -42,24 +43,37 @@ STORAGE_PATH = construct_storage_path()
 
 
 def get_bipartite_graph(topology):
-    ''' Get a graph with Processes, Stores, and edges from a Vivarium topology '''
+    """ Get a graph with Processes, Stores, and edges from a Vivarium topology """
     if 'topology' in topology:
         topology = topology['topology']
 
     process_nodes = []
     store_nodes = []
     edges = {}
+    compartment_nodes = []
+    place_edges = []
     for process_id, connections in topology.items():
-        process_id = process_id.replace("_", "_\n")  # line breaks at underscores
         process_nodes.append(process_id)
 
-        for port, store_id in connections.items():
-            store_id = '\n'.join(store_id)  # TODO: a fancier graph for a dict
-            store_id = store_id.replace('..\n', '⬆︎')
-
+        for port, path in connections.items():
+            # store_id = '\n'.join(path)  # TODO: a fancier graph for a dict
+            # store_id = store_id.replace('..\n', '⬆︎')
+            store_id = path[-1]
             if store_id not in store_nodes:
                 store_nodes.append(store_id)
 
+            if len(path) > 1:
+                # hierarchy place edges between inner/outer stores
+                for store_1, store_2 in zip(path, path[1:]):
+
+                    # save the place edge
+                    place_edge = (store_1, store_2)
+                    place_edges.append(place_edge)
+
+                    # add all intermediate stores to store list
+                    if store_1 not in compartment_nodes:
+                        compartment_nodes.append(store_1)
+
             edge = (process_id, store_id)
             edges[edge] = port
 
@@ -68,12 +82,12 @@ def get_bipartite_graph(topology):
     if overlap:
         print('{} shared by processes and stores'.format(overlap))
 
-    return process_nodes, store_nodes, edges
+    return process_nodes, store_nodes, edges, place_edges
 
 
 def get_networkx_graph(topology):
-    ''' Make a networkX graph from a Vivarium topology '''
-    process_nodes, store_nodes, edges = get_bipartite_graph(topology)
+    """ Make a networkX graph from a Vivarium topology """
+    process_nodes, store_nodes, edges, place_edges = get_bipartite_graph(topology)
 
     # make networkX graph
     g = nx.Graph()
@@ -81,20 +95,26 @@ def get_networkx_graph(topology):
         g.add_node(node_id, type='Process')
     for node_id in store_nodes:
         g.add_node(node_id, type='Store')
+
+    # add topology edges
     for (process_id, store_id), port in edges.items():
         g.add_edge(process_id, store_id, port=port)
 
-    return g
+    return g, place_edges
 
 
 def graph_figure(
         graph: nx.Graph,
         *,
-        graph_format: str = 'bipartite',
+        graph_format: str = 'horizontal',
+        place_edges: Optional[list] = None,
         show_ports: bool = True,
         store_color: Any = 'tab:blue',
         process_color: Any = 'tab:orange',
+        store_colors: Optional[Dict] = None,
+        process_colors: Optional[Dict] = None,
         color_edges: bool = True,
+        edge_width: float = 2.0,
         fill_color: Any = 'w',
         node_size: float = 8000,
         font_size: int = 14,
@@ -106,10 +126,12 @@ def graph_figure(
     """ Make a figure from a networkx graph.
 
     :param graph: the networkx.Graph to plot
-    :param graph_format: 'bipartite' or not
+    :param graph_format: 'horizontal', 'vertical', or 'hierarchy'
     :param show_ports: whether to show the Port labels
-    :param store_color: color for the Store nodes; any matplotlib color value
-    :param process_color: color for the Process nodes; any matplotlib color value
+    :param store_color: default color for the Store nodes; any matplotlib color value
+    :param process_color: default color for the Process nodes; any matplotlib color value
+    :param store_colors: (dict) specific colors for the Store nodes, mapping from store name to matplotlib color
+    :param process_colors: (dict) specific colors for the Process nodes, mapping from store name to matplotlib color
     :param color_edges: color each edge between Store and Process a different color
     :param fill_color: fill color for the Store and Process nodes; any
         matplotlib color value
@@ -121,6 +143,9 @@ def graph_figure(
     :param label_pos: position of the Port labels along their connection lines,
         (0=head, 0.5=center, 1=tail)
     """
+    process_colors = process_colors or {}
+    store_colors = store_colors or {}
+    place_edges = place_edges or []
 
     node_attributes = dict(graph.nodes.data())
     process_nodes = [
@@ -131,48 +156,135 @@ def graph_figure(
         if attributes['type'] == 'Store']
 
     edge_list = list(graph.edges)
-    edges = {
-        edge: graph.edges[edge]['port']
-        for edge in edge_list}
+    edges = {}
+    for edge in edge_list:
+        if 'port' in graph.edges[edge]:
+            edges[edge] = graph.edges[edge]['port']
+
+    # plot
+    n_stores = len(store_nodes)
+    n_processes = len(process_nodes)
+    n_max = max(n_stores, n_processes)
 
     # get positions
     pos = {}
-    if graph_format == 'bipartite':
+    if graph_format == 'hierarchy':
+        # add new place edges by iterating over all place_edges
+        outers = set()
+        inners = set()
+        for (store_1, store_2) in place_edges:
+            outers.add(store_1)
+            inners.add(store_2)
+            graph.add_edge(store_1, store_2, place_edge=True)
+
+        # add non-embedded nodes to outers
+        all_stores = outers.union(inners)
+        non_embedded = set(store_nodes).difference(all_stores)
+        outers.update(non_embedded)
+
+        # add intermediate nodes to store_nodes
+        intermediate_nodes = all_stores.difference(store_nodes)
+        store_nodes.extend(list(intermediate_nodes))
+
+        # determine the hierarchy levels
+        levels = []
+        accounted = set()
+        unaccounted = outers.union(inners)
+        top_level = outers - inners
+        levels.append(list(top_level))
+        accounted.update(top_level)
+        unaccounted = unaccounted.difference(accounted)
+        while len(unaccounted) > 0:
+            next_level = set()
+            for (store_1, store_2) in place_edges:
+                if store_1 in accounted and store_2 in unaccounted:
+                    next_level.add(store_2)
+            levels.append(list(next_level))
+            accounted.update(next_level)
+            unaccounted = unaccounted.difference(accounted)
+
+        # buffer makes things centered
+        n_max = max([len(level) for level in levels])
+        buffer_processes = (n_max - n_processes) / 2
+
+        # place the nodes according to levels
+        for idx, node_id in enumerate(process_nodes, 1):
+            pos[node_id] = np.array([buffer_processes + idx, 1])
+        for level_idx, level in enumerate(levels, 1):
+            level_buffer = (n_max - len(level)) / 2
+            for idx, node_id in enumerate(level, 1):
+                pos[node_id] = np.array([level_buffer + idx, -1*level_idx])
+
+        fig = plt.figure(1, figsize=(n_max * node_distance, 6 + 3 * len(levels)))
+
+    elif graph_format == 'vertical':
+        # processes in a column, and stores in a column
         for idx, node_id in enumerate(process_nodes, 1):
             pos[node_id] = np.array([-1, -idx])
         for idx, node_id in enumerate(store_nodes, 1):
             pos[node_id] = np.array([1, -idx])
 
-    # plot
-    n_rows = max(len(process_nodes), len(store_nodes))
-    fig = plt.figure(1, figsize=(12, n_rows * node_distance))
+        fig = plt.figure(1, figsize=(12, n_max * node_distance))
 
-    # nx.draw(graph, pos=pos, node_size=node_size)
+    elif graph_format == 'horizontal':
+        # processes in a row, and stores in a row
+        # buffer makes things centered
+        buffer_processes = (n_max - n_processes) / 2
+        buffer_stores = (n_max - n_stores) / 2
+
+        for idx, node_id in enumerate(process_nodes, 1):
+            pos[node_id] = np.array([buffer_processes + idx, 1])
+        for idx, node_id in enumerate(store_nodes, 1):
+            pos[node_id] = np.array([buffer_stores + idx, -1])
+
+        fig = plt.figure(1, figsize=(n_max * node_distance, 12))
+
+    # get node colors
+    process_color_list = [
+        process_colors.get(process_name, process_color)
+        for process_name in process_nodes]
+    store_color_list = [
+        store_colors.get(store_name, store_color)
+        for store_name in store_nodes]
+
+    # draw the process nodes
     nx.draw_networkx_nodes(graph, pos,
                            nodelist=process_nodes,
                            node_color=fill_color,
-                           edgecolors=process_color,
+                           edgecolors=process_color_list,
                            node_size=node_size,
                            linewidths=border_width,
-                           node_shape='s'
-                           )
+                           node_shape='s')
+    # draw the store nodes
     nx.draw_networkx_nodes(graph, pos,
                            nodelist=store_nodes,
                            node_color=fill_color,
-                           edgecolors=store_color,
+                           edgecolors=store_color_list,
                            node_size=node_size,
                            linewidths=border_width,
-                           node_shape=cast(str, STORAGE_PATH)
-                           )
+                           node_shape=cast(str, STORAGE_PATH))
+
     # edges
     edge_args = {}
+
+    # edge colors
     if color_edges:
+        edge_args['edge_cmap'] = plt.get_cmap('nipy_spectral')
         edge_args['edge_color'] = list(range(1, len(edges) + 1))
+        if graph_format == 'hierarchy':
+            edge_args['edge_color'].extend([0 for _ in place_edges])
+
+    # edge width
+    edge_args['width'] = [edge_width for _ in edges.keys()]
+    if graph_format == 'hierarchy':
+        # thicker edges for hierarchy connections
+        edge_args['width'].extend([edge_width * 2 for _ in place_edges])
+
     nx.draw_networkx_edges(graph, pos,
-                           width=1.5,
+                           # width=1.5,
                            **edge_args)
 
-    # labels
+    # edge labels
     nx.draw_networkx_labels(graph, pos,
                             font_size=font_size)
     if show_ports:
@@ -181,7 +293,7 @@ def graph_figure(
                                      font_size=font_size,
                                      label_pos=label_pos)
 
-    # add buffer
+    # add white buffer around final figure
     xmin, xmax, ymin, ymax = plt.axis()
     plt.xlim(xmin - buffer, xmax + buffer)
     plt.ylim(ymin - buffer, ymax + buffer)
@@ -198,35 +310,21 @@ def save_network(out_dir='out', filename='network'):
     # plt.close()
 
 
-def plot_compartment_topology(
-        compartment,
-        settings: Optional[Dict[str, Any]] = None,
-        out_dir=None,
-        filename=None,
-):
-    """ 
-    an old function, reproduced by plot_topology """
-    settings = settings or {}
-    return plot_topology(
-        compartment,
-        settings,
-        out_dir,
-        filename)
-
-
 def plot_topology(
         composite,
-        settings: Optional[Dict[str, Any]] = None,
-        out_dir=None,
-        filename=None,
+        settings: Optional[Dict] = None,
+        out_dir: Optional[str] = None,
+        filename: Optional[str] = None,
 ):
     """ Plot a composite's topology """
 
     settings = settings or {}
-    network = composite.generate()
+    if isinstance(composite, Composer):
+        composite = composite.generate()
 
     # make networkx graph
-    g = get_networkx_graph(network)
+    g, place_edges = get_networkx_graph(composite)
+    settings['place_edges'] = place_edges
 
     # make graph figure
     fig = graph_figure(g, **settings)
@@ -298,44 +396,85 @@ class MergePort(Composer):
 
 
 def test_graph(
-        save_fig=False,
-        topology: Optional[Dict[str, Any]] = None
+        fig_name=None,
+        topology=None,
+        settings=None,
 ):
+
     if topology is None:
         topology = {
             'multiport1': {},
             'multiport2': {}}
-    composite = MergePort({'topology': topology})
-    network = composite.generate()
+    composer = MergePort({'topology': topology})
 
-    # make networkx graph
-    g = get_networkx_graph(network)
+    config = {'settings': settings}
+    if fig_name:
+        config.update({
+            'out_dir': 'out/topology',
+            'filename': fig_name})
 
-    # make graph figure
-    fig = graph_figure(g)
+    plot_topology(composer, **config)
 
-    if save_fig:
-        save_network(out_dir='out/topology', filename='topology')
+
+def main():
+
+    parser = argparse.ArgumentParser(description='topology')
+    parser.add_argument(
+        '--topology', '-t',
+        type=str,
+        choices=['1', '2', '3'],
+        help='the topology id'
+    )
+    args = parser.parse_args()
+
+    settings = {}
+    topology_id = str(args.topology)
+    if topology_id == '1':
+        topology = {
+                'multiport1': {
+                    'a': ('D',),
+                    'b': ('D',),
+                    'c': ('D',),
+                },
+                'multiport2': {}}
+        fig_name = 'topology_1'
+        settings = {
+            'graph_format': 'vertical',
+            'process_colors': {'multiport1': 'r'},
+            'store_colors': {'C': 'k'},
+        }
+    elif topology_id == '2':
+        topology = {
+                'multiport1': {
+                    'a': ('A', 'AA',),
+                    'b': ('A', 'BB',),
+                    'c': ('A', 'CC',),
+                },
+                'multiport2': {}}
+        fig_name = 'topology_2'
+    elif topology_id == '3':
+        topology = {
+                'multiport1': {
+                    'a': ('A', 'AA', 'AAA',),
+                    'b': ('A', 'BB',),
+                    'c': ('A', 'CC',),
+                },
+                'multiport2': {}}
+        fig_name = 'topology_3'
+        settings = {
+            'graph_format': 'hierarchy',
+            'store_color': 'navy'
+        }
+    else:
+        pass
+        # more complex topology, with ..?
+
+    test_graph(
+        fig_name=fig_name,
+        topology=topology,
+        settings=settings,
+    )
 
 
 if __name__ == '__main__':
-    # topology = {
-    #         'multiport1': {
-    #             'a': ('D',),
-    #             'b': ('D',),
-    #             'c': ('D',),
-    #         },
-    #         'multiport2': {}}
-
-    topology = {
-            'multiport1': {
-                'a': ('A', 'AA',),
-                'b': ('A', 'BB',),
-                'c': ('A', 'CC',),
-            },
-            'multiport2': {}}
-
-    test_graph(
-        save_fig=True,
-        topology=topology,
-    )
+    main()
